
/* Medusa DS9 header file
 * by Marek Zelem
 */

#ifndef _MEDUSA_H_
#define _MEDUSA_H_

#define __KERNEL__

#define _LINUX_FS_H 1
#include <linux/capability.h>
#undef _LINUX_FS_H
#include <linux/kdev_t.h>

#undef __KERNEL__

#include <linux/sys.h>
#include <linux/types.h>

/* extension for struct task_struct */
struct med_proc {
	__u16 med_vs;		/* bitmap of 16 virtual spaces (tie vidi) */
	__u16 med_vs_sw;	/* maska, ktore moze prepinat (for med_vs) */
	__u16 med_m_vs;		/* bitmap, do ktorych moze modifik. (chmody) */
	__u16 med_w_vs;		/* bitmap, ktore moze prepisovat/zapisovat */
	__u16 med_act;		/* akcie ktore vyzaduju potvrdenie od constabla */
	__u16 med_iact;		/* iakcie ktore vyzaduju potvrdenie od constabla */
	__u32 med_user;		/* for free deamon use */
#ifdef CONFIG_MEDUSA_SYSCALL
	__u32 med_syscall[NR_syscalls / 32];	/* bitmap syscallov, ktore sa oznamuju */
#endif
#ifdef CONFIG_MEDUSA_FORCE
	void *force_code;	/* Kod na vnutenie alebo NULL, kfree */
	int force_len;		/* dlzka kodu na vnutenie */
#endif
};

/* rozsirenie struct inode */
struct med_inode {
	__u16 med_vs;		/* bitmap of 16 virtual spaces */
	__u16 med_act;		/* akcie ktore vyzaduju potvrdenie od constabla */
	u_long cinfo;		/* info for constable */
	__u16 magic;		/* ci je aktualny */
};

extern struct task_struct *constable;
int medusa_capable(int cap);

#define	BEGIN_VS	(0xffff)

struct iattr;

/* process akcie */
#define	MPACT_FORK	0x0001	/* OK (clone_flags) */
#define	MPACT_EXEC	0x0002	/* OK (test-0 execnute-1) */
#define	MPACT_EXIT	0x0004
#define	MPACT_SETUID	0x0008	/* OK (uid) setuid, setreuid, !setgid, !setgroups ... */
#define	MPACT_KILL	0x0010	/* OK sendsig */
#define MPACT_SOCKET	0x0020	/* socketcall ... */
#define	MPACT_PERM	0x0100	/* je vo funkcii permission */
#define	MPACT_CH	0x0200	/* je vo funkcii notify_change */
#define	MPACT_IOP	0x0400	/* OK robi inode operation (MIACT_) */
#define	MPACT_CAP	0x0800	/* OK (cap) capable, na ktore nama pravo */
#define	MPACT_MODULE	0x1000	/* create_module,init_module,delete_module */
#define	MPACT_SEXEC	0x2000	/* OK target=nove uid, inof1=vs inodu info2=cinfo */
#define	MPACT_MAY_CD	0x8000	/* OK docastne, moze sa prepinat do vsetkych */
				/* adresarov */

struct m_proc_inf {
	int pid, parent_pid, child_pid, sibling_pid;	/* older sib. */
	int pgrp;
	unsigned short uid, euid, suid, fsuid;
	unsigned short gid, egid, sgid, fsgid;
	unsigned short luid;
	kernel_cap_t cap_effective, cap_inheritable, cap_permitted;
	struct med_proc mp;
};

/* inode akcie  +YES -ak je odpoved YES tak sa operacia nevykona a nebude err */
#define	MIACT_ACCESS	0x0001	/* OK (inode, 0, NULL) - vzdy ked najde tento
				   inod, moze presmerovat */
#define	MIACT_CREATE	0x0002	/* OK (dir, mode, meno) */
#define	MIACT_LINK	0x0004	/* OK (inode, *newname, meno +YES */
#define	MIACT_UNLINK	0x0008	/* OK (inode, 0, meno) +YES */
#define	MIACT_SYMLINK	0x0010	/* OK (dir, *oldname, meno) +YES */
#define	MIACT_MKDIR	0x0020	/* OK (dir, mode, meno) +YES */
#define	MIACT_RMDIR	0x0040	/* OK (inode, 0, meno) +YES */
#define	MIACT_MKNOD	0x0080	/* OK (dir, dev, meno) */
#define	MIACT_RENAME	0x0100	/* OK (inode, *newname, meno) +YES */
#define	MIACT_READLINK	0x0200	/* !!! zatial nie, ale moze sa hodit */
#define	MIACT_FOLLOW	0x0400	/* !!! naco ??? */
#define	MIACT_TRUNCATE	0x0800	/* OK (inode, length, meno) +YES */
#define	MIACT_PERMISSION	0x1000	/* OK (inode, mask, meno) +YES */
#define	MIACT_EXEC	0x2000	/* OK (inode, argv,envp, path), redir */
#define	MIACT_INHERIT	0x4000	/* OK - vlastnosti inodu sa dedia */
#define	MIACT_VALID	0x8000	/* OK - informacia je platna */

struct m_inode_inf {
	kdev_t dev;
	unsigned long ino;
	__kernel_mode_t mode;
	__kernel_nlink_t nlink;
	__kernel_uid_t uid;
	__kernel_gid_t gid;
	kdev_t rdev;
	struct med_inode mi;
};

/* cmd */
#define	MED_VERSION		0
#define	MED_IGET	1	/* Nacitanie informacii k inodu */
struct medreq_iget {
	struct m_inode_inf inode;
	struct m_inode_inf parent;
	/* + meno */
};
struct medans_iget {
	struct m_inode_inf inode;
};

#define	MED_IACT	2	/* Oznamenie akcie inodu */
struct medreq_iact {
	__u16 pact;		/* akcia procesu (0= inode action/ proc.act) */
	__u16 act;		/* sub-akcia ktoru sa snazi proces vykonat */
	struct m_inode_inf inode;
	struct m_proc_inf proc;
	__u32 info1;		/* doplnkova informacia */
	__u32 info2;		/* doplnkova informacia */
	/* + data (meno) */
};
struct medans_iact {
	struct m_proc_inf proc;
	/* ... ??? */
	/* + presmerovanie(pathname) */
};

#define	MED_PACT	3	/* Oznamenie akcie procesu */
struct medreq_pact {
	__u16 act;		/* akcia ktoru sa snazi proces vykonat */
	struct m_proc_inf proc;
	struct m_proc_inf target;
	__u32 info1;		/* doplnkova informacia */
	__u32 info2;		/* doplnkova informacia */
};
struct medans_pact {
	struct m_proc_inf proc;
};

#define	MED_TRACE	4	/* Oznamenie syscallu */
struct medreq_trace {
	__u16 syscall;		/* syscall */
	struct m_proc_inf proc;
	__u32 arg[5];
};
struct medans_trace {
	struct m_proc_inf proc;
	__u32 arg[5];
	__u32 retval;		/* platne len pri odpovedi MED_NOT */
};

/* prikazy cmd kernelu od daemona */
#define	MED_RESET	0x100
#define	MED_GET_PROC	0x101
#define	MED_SET_PROC	0x102
#define	MED_FORCE	0x103
#define	MED_GETSTR	0x110
#define	MED_PRINTK	0x111
struct medreq_cmd {
	__u32 arg[8];
	struct m_proc_inf proc;
};
struct medans_cmd {
	__u32 arg[8];
	struct m_proc_inf proc;
};

/* answer */
#define	MED_ERR	-1		/* error */
#define MED_OK	0		/* Ok - answer data valid */
#define MED_YES	1		/* Ok - answer data valid + all done => do nothing */
#define	MED_NOT 2		/* permision denied - answer data valid */

struct medusa_packet {
	int id;
	int cmd;
	int answer;
	int data_len;		/* kernel->daemon, daemon->kernel */
	union {
		struct medreq_iget r_iget;
		struct medans_iget a_iget;
		struct medreq_iact r_iact;
		struct medans_iact a_iact;
		struct medreq_pact r_pact;
		struct medans_pact a_pact;
		struct medreq_trace r_trace;
		struct medans_trace a_trace;
		struct medreq_cmd r_cmd;
		struct medans_cmd a_cmd;
	} u;
};

static inline int med_c_valid(struct med_inode *mi)
{
	return ((mi->med_act & MIACT_VALID) ? 1 : 0);
}

#endif				/* _MEDUSA_H_ */
